package cn.com.ty.lift.business.annualinspection.controller;


import cn.com.ty.lift.business.annualinspection.dto.InspectionRequest;
import cn.com.ty.lift.business.annualinspection.dto.InspectionResponse;
import cn.com.ty.lift.business.annualinspection.entity.AnnualInspection;
import cn.com.ty.lift.business.annualinspection.service.AnnualInspectionService;
import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.common.utils.AliyunOSSUtil;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.time.LocalDateTime;

/**
 * <p>
 * 年检记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-10
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/annualinspection/annual-inspection")
public class AnnualInspectionController {

    private AnnualInspectionService annualInspectionService;

    private SystemConfiguration systemConfiguration;

    private AliyunOSSUtil aliyunOSSUtil;

    /**
     * @description 根据id查询
     * @date 2019/12/10 10:19
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody InspectionRequest request){
        Long id = request.getId();
        if(null == id || id <= 0){
            return RestResponse.failParam();
        }

        InspectionResponse entity = annualInspectionService.selectByIdWithInfo(request);
        if(null == entity){
            return RestResponse.fail();
        }
        return RestResponse.success(entity);
    }

    /**
     * @description 根据条件查询年检记录
     * @date 2019/12/11 17:15
     * @param
     * @return
     */
    @PostMapping("page")
    public RestResponse page(@RequestBody InspectionRequest request){
        if(null == request){
            return RestResponse.failParam();
        }
        String info = request.getInfo();
        if(StrUtil.isNotBlank(info)){
            request.setInfo(StrUtil.format("%{}%",info));
        }

        IPage<InspectionResponse> pages = annualInspectionService.pageByCondition(request);
        if(null == pages){
            return RestResponse.fail();
        }
        long count = annualInspectionService.countByCondition(request);
        CountPage<InspectionResponse> countPage = new CountPage<>(count, pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 修改年检信息
     *
     * status 状态（0 待完成；1 已完成；2 超期）
     *
     * stepStatus  :年检阶段状态
     *
     * 未开始0，第一阶段 1-2，第二阶段 3，第三阶段 4-7，第四阶段 8-10
     *
        1  维保工确认年检计划
        2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
            文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
        --------------------------------
        3  企业文员确认现场检验时间
            录入人员信息，政府质检，企业质检，联系电话
        --------------------------------
        4	维保工上传检验结果
        5	企业文员确认检验结果-合格 --> 不能算年检超期
        6	企业文员确认检验结果-整改 --> 回到第二阶段
        7	企业文员确认检验结果-不合格 --> 可以算年检超期
        --------------------------------
        8	维保工上传年检报告和合格证
        9	企业文员确认年检报告和合格证-合格
        10	企业文员确认年检报告和合格证-不合格 -->停用电梯
     * @date 2019/12/10 10:04
     * @param
     * @return
     */
    @PostMapping("modify")
    public RestResponse modify(@RequestBody AnnualInspection entity){
        if(null == entity){
            return RestResponse.failParam();
        }

        Long id = entity.getId();
        if(null == id || id <= 0){
            return RestResponse.failParam();
        }
        AnnualInspection old = annualInspectionService.getById(id);
        if(null == old){
            return RestResponse.failParam();
        }
        Integer oldStepStatus = old.getStepStatus();
        Integer stepStatus = entity.getStepStatus();

        //1 维保工确认年检计划
        if(oldStepStatus == Values.AI_STATUS_NOT_START && (stepStatus != Values.AI_STATUS_NOT_START && stepStatus != Values.AI_STATUS_COMFIRM)){
            return RestResponse.fail("请执行[维保工确认年检计划]操作");
        }
        /**
         * 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
         *             文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
         */
        if(oldStepStatus == Values.AI_STATUS_COMFIRM && stepStatus != Values.AI_STATUS_SELFCHECK){
            return RestResponse.fail("请执行[维保工上传自检报告]操作");
        }
        if(stepStatus == Values.AI_STATUS_SELFCHECK && StrUtil.isEmpty(entity.getSelfcheckReportImg())){
            return RestResponse.fail("请执行[上传自检报告]操作");
        }
        if(stepStatus == Values.AI_STATUS_SELFCHECK && ObjectUtil.isEmpty(entity.getSelfcheckDate())){
            return RestResponse.fail("请确认自检时间");
        }
        //3  企业文员确认现场检验时间，录入人员信息，政府质检，企业质检，联系电话
        if(oldStepStatus == Values.AI_STATUS_SELFCHECK && stepStatus != Values.AI_STATUS_CHECK_COMFIRM){
            return RestResponse.fail("请执行[确认现场检验时间]操作");
        }
        if (stepStatus == Values.AI_STATUS_CHECK_COMFIRM){
            if(ObjectUtil.isEmpty(entity.getCheckDate())){
                return RestResponse.fail("请设置现场检验时间");
            }
            if(ObjectUtil.isEmpty(entity.getInspector())){
                return RestResponse.fail("请设置政府质检人员");
            }
            if(ObjectUtil.isEmpty(entity.getOperator())){
                return RestResponse.fail("请设置公司质检人员");
            }
            if(ObjectUtil.isEmpty(entity.getTelephone())){
                return RestResponse.fail("请设置人员联系电话");
            }

        }
        // 4  维保工上传检验结果
        if(oldStepStatus == Values.AI_STATUS_CHECK_COMFIRM && stepStatus != Values.AI_STATUS_CHECK_UPLOAD){
            return RestResponse.fail("请执行[上传检验结果]操作");
        }
        if(stepStatus == Values.AI_STATUS_CHECK_UPLOAD && StrUtil.isEmpty(entity.getCheckResultImg())){
            return RestResponse.fail("请执行[上传检验结果]操作");
        }
        /**
         *  5	企业文员确认检验结果-合格
         *  6	企业文员确认检验结果-整改
         *  7	企业文员确认检验结果-不合格
         */
        if(oldStepStatus == Values.AI_STATUS_CHECK_UPLOAD && (stepStatus != Values.AI_STATUS_CHECK_QUALIFIED && stepStatus != Values.AI_STATUS_CHECK_RECTIFICATION && stepStatus !=Values.AI_STATUS_CHECK_UNQUALIFIED)){
            return RestResponse.fail("请执行[企业文员确认检验结果]操作");
        }
        /**
         * 5	企业文员确认检验结果-合格 --> 8	维保工上传年检报告和合格证
         * 7	企业文员确认检验结果-不合格-->8	维保工上传年检报告和合格证
         */
        if((oldStepStatus == Values.AI_STATUS_CHECK_QUALIFIED || oldStepStatus == Values.AI_STATUS_CHECK_UNQUALIFIED) && stepStatus != Values.AI_STATUS_CERTIFICATE_REPORT_UPLOAD){
            return RestResponse.fail("请执行[维保工上传年检报告和合格证]操作");
        }

        // 6	企业文员确认检验结果-整改 --> 4	维保工上传检验结果
        if(oldStepStatus == Values.AI_STATUS_CHECK_RECTIFICATION && stepStatus != Values.AI_STATUS_CHECK_UPLOAD){
            return RestResponse.fail("请执行[维保工上传检验结果]操作");
        }
        /**
         * 8	维保工上传年检报告和合格证--> 9  企业文员确认年检报告和合格证 -合格
         * 8	维保工上传年检报告和合格证--> 10  企业文员确认年检报告和合格证 -不合格
         */
        if(oldStepStatus == Values.AI_STATUS_CERTIFICATE_REPORT_UPLOAD && (stepStatus != Values.AI_STATUS_CERTIFICATE_REPORT_QUALIFIED && stepStatus != Values.AI_STATUS_CERTIFICATE_REPORT_UNQUALIFIED)){
            return RestResponse.fail("请执行[企业文员确认年检报告和合格证]操作");
        }
        //合格：文员确认年检报告并设置下次年检时间
        if(stepStatus == Values.AI_STATUS_CERTIFICATE_REPORT_QUALIFIED && null == entity.getNextInspectionTime()){
            return RestResponse.fail("请设置下次年检时间");
        }

        int stepName = Values.toStepName(stepStatus);
        if(stepName == -1){
            return RestResponse.fail("年检状态有误,状态码: " + stepStatus);
        }
        entity.setStepName(stepName);

        if(stepStatus == Values.AI_STATUS_CERTIFICATE_REPORT_QUALIFIED || stepStatus == Values.AI_STATUS_CERTIFICATE_REPORT_UNQUALIFIED){
            // 设置年检完成 status = 1
            entity.setFinishTime(LocalDateTime.now());
            entity.setStatus(1);
        }


        boolean result = annualInspectionService.saveOrUpdate(entity);
        if(result){
            //todo 不合格：流程结束，电梯扭转到不合格状态列表，电梯停保，清理维保计划记录
            if(stepStatus == Values.AI_STATUS_CERTIFICATE_REPORT_UNQUALIFIED){

            }
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    @GetMapping("show")
    public BufferedImage show(@RequestParam("filename") String filename) throws IOException {
        if(StrUtil.isEmpty(filename)){
            return null;
        }
        String fileurl = aliyunOSSUtil.getObjectUrl(systemConfiguration.bucketName,filename);
        log.info("fileurl: " + fileurl);
        File file = new File("download/" + filename);
        File down = aliyunOSSUtil.getFile(systemConfiguration.bucketName,filename,file);
        if(null == down){
            return null;
        }
        return ImageIO.read(new FileInputStream(file));
    }
    /**
     * 其他角色获取自己关联的项目的年检任务
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody InspectionRequest request){
        IPage<InspectionResponse> pages = annualInspectionService.pageByUser(request);
        if(null == pages){
            return RestResponse.fail();
        }
        return RestResponse.success(pages);
    }

    /**
     * 确认年检计划，传递年检id，后台修改年检状态，停止信息推送
     * @param request
     * @return
     */
    @PostMapping("confirm")
    public RestResponse confirm(@RequestBody InspectionRequest request){
        Long id = request.getId();
        if(null == id || id < 0){
            return RestResponse.failParam();
        }
        AnnualInspection entity = annualInspectionService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setStepStatus(Values.AI_STATUS_COMFIRM);
        entity.setStepName(Values.AI_STEP_ONE);
        boolean result = annualInspectionService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * 上传自检报告，传递年检id，传递自检报告图片路径
     * @return
     */
    @PostMapping("selfcheck")
    public RestResponse selfcheck(@RequestBody InspectionRequest request){
        Long id = request.getId();
        if(null == id || id < 0){
            return RestResponse.failParam();
        }
        LocalDateTime selfcheckDate = request.getSelfcheckDate();
        String selfcheckReportImg = request.getSelfcheckReportImg();
        if(null == selfcheckDate || null == selfcheckReportImg){
            return RestResponse.fail("缺少自检信息");
        }
        AnnualInspection entity = annualInspectionService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setSelfcheckDate(selfcheckDate);
        entity.setSelfcheckReportImg(selfcheckReportImg);
        entity.setStepStatus(Values.AI_STATUS_SELFCHECK);
        entity.setStepName(Values.AI_STEP_ONE);
        boolean result = annualInspectionService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * 上传检验结果，传递年检id，传递检验报告图片路径
     * @param request
     * @return
     */
    @PostMapping("check")
    public RestResponse check(@RequestBody InspectionRequest request){
        Long id = request.getId();
        if(null == id || id < 0){
            return RestResponse.failParam();
        }
        LocalDateTime checkDate = request.getCheckDate();
        String checkResultImg = request.getCheckResultImg();
        if(null == checkDate || null == checkResultImg){
            return RestResponse.fail("缺少检验信息");
        }
        AnnualInspection entity = annualInspectionService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setCheckDate(checkDate);
        entity.setCheckResultImg(checkResultImg);
        entity.setStepStatus(Values.AI_STATUS_CHECK_UPLOAD);
        entity.setStepName(Values.AI_STEP_THREE);
        boolean result = annualInspectionService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * 上传年检报告和合格证，传递年检id，传递年检报告和合格证图片路径
     * @param request
     * @return
     */
    @PostMapping("finish")
    public RestResponse finish(@RequestBody InspectionRequest request){
        Long id = request.getId();
        if(null == id || id < 0){
            return RestResponse.failParam();
        }
        String certificateImgUrl = request.getCertificateImgUrl();
        if(StrUtil.isEmpty(certificateImgUrl)){
            return RestResponse.fail("缺少合格证信息");
        }
        String annualInspectionImg = request.getAnnualInspectionImg();
        if(StrUtil.isEmpty(annualInspectionImg)){
            return RestResponse.fail("缺少年检报告信息");
        }
        AnnualInspection entity = annualInspectionService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setCertificateImgUrl(certificateImgUrl);
        entity.setAnnualInspectionImg(annualInspectionImg);
        entity.setStepStatus(Values.AI_STATUS_CERTIFICATE_REPORT_UPLOAD);
        entity.setStepName(Values.AI_STEP_FOUR);
        boolean result = annualInspectionService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }
}
